#include <IRremote.h>  //Importar a IRremote library.
#include <EEPROM.h> //Importar a EEPROM library.

int RECV_PIN = 8; //Variável para o pino do receptor IR.
IRrecv Receptor_IR(RECV_PIN); //Definir o pino de recepção de sinal.
decode_results Leitura_Codigo;  //Variável para representar o código do sinal na recepção.

IRsend Emissor_IR;  //Variável para representar o código do sinal na emissão.

long unsigned int Codigo; //Variável para captação do código IR.

//Variáveis para fragmentar o código de 32bits em partes de 8bits:
byte Byte_1;
byte Byte_2;
byte Byte_3;
byte Byte_4;
int Protocolo;

//Variáveis para gravação e leitura do código na EEPROM:
long unsigned int Parte_1;
long unsigned int Parte_2;
long unsigned int Final;

//Variável para o endereço na EEPROM:
int Endereco_EEPROM = 0;

//Variáveis para selecção do pino de entrada dos botões:
int Pino_Botao_Gravar = 13;
int Pino_Botao_1 = 11;
int Pino_Botao_2 = 12;

//Correr a função SETUP apenas uma vez após ser pressionado Reset:
void setup() {
  //Iniciar a  Communication Serial a 9600b/s:
  Serial.begin(9600);
  
  //Iniciar a função de recepção de sinal IR:
  Receptor_IR.enableIRIn();

  //Definição do tipo de pino (Saída ou Entrada):
  pinMode(Pino_Botao_Gravar, INPUT);
  pinMode(Pino_Botao_1, INPUT);
  pinMode(Pino_Botao_2, INPUT);
}
//Correr a função LOOP repetidamente:
void loop() {
  //Condição para ler o sinal IR e guarda-lo na EEPROM em cada um dos botões:
  if (digitalRead(Pino_Botao_Gravar) == HIGH && digitalRead(Pino_Botao_1) == HIGH || digitalRead(Pino_Botao_Gravar) == HIGH && digitalRead(Pino_Botao_2) == HIGH) {
  
    //Escreve no Monitor Serial:
    Serial.println("Procurando sinal...");
    Codigo = 0; //Coloca novamente a variável a 0:
    
    //Verifica se foi recebido um código IR:
    if(Receptor_IR.decode(&Leitura_Codigo)) {
      //Verifica se o código recebido é um código de repetição.
      if(Leitura_Codigo.value != 0xFFFFFFFF) {
        //Coloca na variável o código recebido.
        Codigo = Leitura_Codigo.value;
      }
      else {
        //Coloca na variável o código recebido.
        Leitura_Codigo.value = Codigo;
      }
      //Escreve no Monitor Serial:
      Serial.println("Codigo recebido:");
      Serial.println(Codigo, HEX);

      //Fragmentar o código de 32bits em partes de 8bits:
      Byte_1 = (Codigo >> 24) & 0xff;
      Byte_2 = (Codigo >> 16) & 0xff;
      Byte_3 = (Codigo >> 8) & 0xff;
      Byte_4 = (Codigo >> 0) & 0xff; //ou (Codigo) & 0xff; <<< Ambas as expressões são válidas.

      //Condição para verificar está activo o botão 1:
      if (digitalRead(Pino_Botao_1) == HIGH) {
        //Coloca na variável o 1º endereço da EEPROM para o botão 1.
        Endereco_EEPROM = 1;
      }
      //Condição para verificar está activo o botão 2:
      else if (digitalRead(Pino_Botao_2) == HIGH) {
        //Coloca na variável o 1º endereço da EEPROM para o botão 2.
        Endereco_EEPROM = 6;
      }

      //Gravar na EEPROM um fragmento do código recebido:
      EEPROM.write(Endereco_EEPROM, Byte_1);
      //Altera o endereço da EEPROM:
      Endereco_EEPROM = ++Endereco_EEPROM;
      //Gravar na EEPROM um fragmento do código recebido:
      EEPROM.write(Endereco_EEPROM, Byte_2);
      //Altera o endereço da EEPROM:
      Endereco_EEPROM = ++Endereco_EEPROM;
      //Gravar na EEPROM um fragmento do código recebido:
      EEPROM.write(Endereco_EEPROM, Byte_3);
      //Altera o endereço da EEPROM:
      Endereco_EEPROM = ++Endereco_EEPROM;
      //Gravar na EEPROM um fragmento do código recebido:
      EEPROM.write(Endereco_EEPROM, Byte_4);

      //Altera o endereço da EEPROM:
      Endereco_EEPROM = ++Endereco_EEPROM;
      
      //Condição para determinar o tipo de protocolo:
      switch (Leitura_Codigo.decode_type) {
      
        case NEC:
          //Escreve no Monitor Serial:
          Serial.println("NEC");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 1;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case SONY:
          //Escreve no Monitor Serial:
          Serial.println("SONY");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 2;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case RC5:
          //Escreve no Monitor Serial:
          Serial.println("RC5");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 3;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case RC6:
          //Escreve no Monitor Serial:
          Serial.println("RC6");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 4;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case DISH:
          //Escreve no Monitor Serial:
          Serial.println("DISH");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 5;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case SHARP:
          //Escreve no Monitor Serial:
          Serial.println("SHARP");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 6;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case JVC:
          //Escreve no Monitor Serial:
          Serial.println("JVC");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 7;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case SANYO:
          //Escreve no Monitor Serial:
          Serial.println("SANYO");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 8;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case MITSUBISHI:
          //Escreve no Monitor Serial:
          Serial.println("MITSUBISHI");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 9;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case SAMSUNG:
          //Escreve no Monitor Serial:
          Serial.println("SAMSUNG");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 10;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case LG:
          //Escreve no Monitor Serial:
          Serial.println("LG");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 11;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case WHYNTER:
          //Escreve no Monitor Serial:
          Serial.println("WHYNTER");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 12;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case AIWA_RC_T501:
          //Escreve no Monitor Serial:
          Serial.println("AIWA_RC_T501");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 13;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case PANASONIC:
          //Escreve no Monitor Serial:
          Serial.println("PANASONIC");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 14;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        case DENON:
          //Escreve no Monitor Serial:
          Serial.println("DENON");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 15;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        default:
        case UNKNOWN:
          //Escreve no Monitor Serial:
          Serial.println("UNKNOWN");
          //Coloca na variável Protocolo o numero relativo:
          Protocolo = 0;
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, Protocolo);
          break;
        }

        //Escreve no Monitor Serial:
        Serial.println("Protocolo:");

        //Verifica se o código recebido é valido:
        if(Leitura_Codigo.value == 0) {
          //Gravar na EEPROM o tipo protocolo recebido:
          EEPROM.write(Endereco_EEPROM, 0);
          //Escreve no Monitor Serial:
          Serial.println("ERRO NO NA LEITURA DO CÓDIGO:");
          //Pausa entre captações de sinal IR.
          delay(1000);
        }
      }
    
    Endereco_EEPROM = 0;  //Coloca novamente o endereço da EEPROM a 0:
  }
  //Condição para ler da EEPROM e emitir o sinal IR no botão 1:
  else if (digitalRead(Pino_Botao_Gravar)== LOW && digitalRead(Pino_Botao_1) == HIGH) {
    
    Endereco_EEPROM = 1;  //Coloca o endereço da EEPROM com o valor 1:
    
    //Lê da EEPROM um fragmento do código:
    Byte_1 = EEPROM.read(Endereco_EEPROM);
    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Byte_2 = EEPROM.read(Endereco_EEPROM);
    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Byte_3 = EEPROM.read(Endereco_EEPROM);
    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Byte_4 = EEPROM.read(Endereco_EEPROM);
    
    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Protocolo = EEPROM.read(Endereco_EEPROM);

    //Escreve no Monitor Serial:
    Serial.println("Codigo Botao 1:");
    
    //Junta os fragmento do código de 8bits em uma parte de 16bits:
    Parte_1 = (((Byte_2 << 0) & 0xFF) + ((Byte_1 << 8) & 0xFFFF));
    //Junta os fragmento do código de 8bits em uma parte de 16bits:
    Parte_2 = (((Byte_4 << 0) & 0xFF) + ((Byte_3 << 8) & 0xFFFF));
    //Junta os fragmento do código de 16bits no código final de 32bits:
    Final = (Parte_1 << 16) + Parte_2;
      
    //Escreve no Monitor Serial:
    Serial.println(Final, HEX);
    //Condição para emitir o sinal IR no protocolo indicado:
    if (Protocolo == 1) {
      //Emite o sinal IR:
      Emissor_IR.sendNEC(Final, 32);
    }
    else if (Protocolo == 2) {
      //Emite o sinal IR:
      Emissor_IR.sendSony(Final, 12);
    }
    else if (Protocolo == 3) {
      //Emite o sinal IR:
      Emissor_IR.sendRC5(Final, 12);
    }
    else if (Protocolo == 4) {
      //Emite o sinal IR:
      Emissor_IR.sendRC6(Final, 20);
    }
    else if (Protocolo == 5) {
      //Emite o sinal IR:
      Emissor_IR.sendDISH(Final, 16);
    }
    else if (Protocolo == 6) {
      //Emite o sinal IR:
      Emissor_IR.sendSharp(Final, 15);
    }
    else if (Protocolo == 7) {
      //Emite o sinal IR:
      //Emissor_IR.sendJVC(Final,16); <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 8) {
      //Emite o sinal IR:
      //Emissor_IR.sendSanyo(Final, 12);  <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 9) {
      //Emite o sinal IR:
      //Emissor_IR.sendMITSUBISHI(Final, 16); <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 10) {
      //Emite o sinal IR:
      Emissor_IR.sendSAMSUNG(Final, 32);
    }
    else if (Protocolo == 11) {
      //Emite o sinal IR:
      Emissor_IR.sendLG(Final, 28);
    }
    else if (Protocolo == 12) {
      //Emite o sinal IR:
      Emissor_IR.sendWhynter(Final, 32);
    }
    else if (Protocolo == 13) {
      //Emite o sinal IR:
      //Emissor_IR.sendAiwaRCT501(Final, 26); <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 14) {
      //Emite o sinal IR:
      Emissor_IR.sendPanasonic(Final, 48);
    }
    else if (Protocolo == 15) {
      //Emite o sinal IR:
      Emissor_IR.sendDenon(Final, 32);
    }

    //Escreve no Monitor Serial:
    Serial.println(Protocolo);

    //Iniciar a função de recepção de sinal IR:
    Receptor_IR.enableIRIn();
  }
  
  //Condição para ler da EEPROM e emitir o sinal IR no botão 2:
  else if (digitalRead(Pino_Botao_Gravar) == LOW && digitalRead(Pino_Botao_2) == HIGH) {
    
    Endereco_EEPROM = 6;  //Coloca o endereço da EEPROM com o valor 6:
    
    //Lê da EEPROM um fragmento do código:
    Byte_1 = EEPROM.read(Endereco_EEPROM);
    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Byte_2 = EEPROM.read(Endereco_EEPROM);
    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Byte_3 = EEPROM.read(Endereco_EEPROM);
    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Byte_4 = EEPROM.read(Endereco_EEPROM);

    //Altera o endereço da EEPROM:
    Endereco_EEPROM = ++Endereco_EEPROM;
    //Lê da EEPROM um fragmento do código:
    Protocolo = EEPROM.read(Endereco_EEPROM);

    //Escreve no Monitor Serial:
    Serial.println("Codigo Botao 2:");

    //Junta os fragmento do código de 8bits em uma parte de 16bits:
    Parte_1 = (((Byte_2 << 0) & 0xFF) + ((Byte_1 << 8) & 0xFFFF));
    //Junta os fragmento do código de 8bits em uma parte de 16bits:
    Parte_2 = (((Byte_4 << 0) & 0xFF) + ((Byte_3 << 8) & 0xFFFF));
    //Junta os fragmento do código de 16bits no código final de 32bits:
    Final = (Parte_1 << 16) + Parte_2;

    //Escreve no Monitor Serial:
    Serial.println(Final, HEX);

    //Condição para emitir o sinal IR no protocolo indicado:
    if (Protocolo == 1) {
      //Emite o sinal IR:
      Emissor_IR.sendNEC(Final, 32);
    }
    else if (Protocolo == 2) {
      //Emite o sinal IR:
      Emissor_IR.sendSony(Final, 12);
    }
    else if (Protocolo == 3) {
      //Emite o sinal IR:
      Emissor_IR.sendRC5(Final, 12);
    }
    else if (Protocolo == 4) {
      //Emite o sinal IR:
      Emissor_IR.sendRC6(Final, 20);
    }
    else if (Protocolo == 5) {
      //Emite o sinal IR:
      Emissor_IR.sendDISH(Final, 16);
    }
    else if (Protocolo == 6) {
      //Emite o sinal IR:
      Emissor_IR.sendSharp(Final, 15);
    }
    else if (Protocolo == 7) {
      //Emite o sinal IR:
      //Emissor_IR.sendJVC(Final,16); <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 8) {
      //Emite o sinal IR:
      //Emissor_IR.sendSanyo(Final, 12);  <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 9) {
      //Emite o sinal IR:
      //Emissor_IR.sendMITSUBISHI(Final, 16); <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 10) {
      //Emite o sinal IR:
      Emissor_IR.sendSAMSUNG(Final, 32);
    }
    else if (Protocolo == 11) {
      //Emite o sinal IR:
      Emissor_IR.sendLG(Final, 28);
    }
    else if (Protocolo == 12) {
      //Emite o sinal IR:
      Emissor_IR.sendWhynter(Final, 32);
    }
    else if (Protocolo == 13) {
      //Emite o sinal IR:
      //Emissor_IR.sendAiwaRCT501(Final, 26); <<< Esta biblioteca não permite emitir neste protocolo.
    }
    else if (Protocolo == 14) {
      //Emite o sinal IR:
      Emissor_IR.sendPanasonic(Final, 48);
    }
    else if (Protocolo == 15) {
      //Emite o sinal IR:
      Emissor_IR.sendDenon(Final, 32);
    }
    
    //Escreve no Monitor Serial:
    Serial.println(Protocolo);

    //Iniciar a função de recepção de sinal IR:
    Receptor_IR.enableIRIn();
  }
  //Termina a captação de sinal IR.
  Receptor_IR.resume();

  //Tempo decorrido entre captações e emissões de sinal IR.
  delay(150);
}
